;**********************************************************************
;                                                                     *
;    Filename:	    FL7-13H.asm                                       *
;    Date:          05/28/09                                          *
;    File Version:  00                                                *
;    Checksum:		0x                                                * 
;                                                                     *
;    Author:        Paul E. Schoen                                    *
;    Company:       P S Technology, Inc.                              *
;                                                                     *
;**********************************************************************
;    Notes:                                                           *
; 05/28/09 -  - Create from FL7-13-200
; 
;**********************************************************************
; PWM inhibited if Vin < MINVIN

	list		p=16HV616		; list directive to define processor
	#include	<P16HV616.inc>		; processor specific variable definitions
	
	__CONFIG    _CP_OFF & _BOD_ON & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_ON
;	__CONFIG    _CP_OFF & _CPD_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _FCMEN_OFF & _IESO_OFF

#define	BANK1		banksel 0x80	;Select BANK1, STATUS<5> = 1
#define	BANK0		banksel 0x00	;Select BANK0, STATUS<5> = 0


;PWM frequency may be 2.98 (3), 11.90 (12), or 47.62 kHz (48)
#define		PWMFREQ		11

; Actual A/D values for measured inputs

; VIN: 8.53 counts per volt: 8V = 68, 9V = 77, 10V = 85, 15V = 128, 16V = 136
#define		MINVIN		D'68'	;	=(9*256)/30		, Actual Vin * MaxADC(256) / Vref(5) / Ratio(6)
#define		MAXVIN		D'136'	;	=(16*256)/30	, Actual Vin * MaxADC(256) / Vref(5) / Ratio(6)

;Bit 7=0 for left justified, bit 6 = 0 for Vdd ref
#define		SEL_AD0		b'00000101'				;001 = AD1
#define		SEL_AD3		b'00001101'				;011 = AD3
#define		SEL_AD4		b'00010001'				;100 = AD4
#define		SEL_AD5		b'00010101'				;101 = AD5
#define		SEL_AD6		b'00011001'				;110 = AD6
#define		SEL_AD7		b'00011101'				;111 = AD7

; ***** PORTA definitions *****
AD_ISW		EQU		ANS0		;[13] RA0, AN0 or C1IN+ (CIS=1), MOSFET current 1A = 100 mV, ICSP Data (Open JP1 to program) ***Not Used***
ICSPC		EQU		1			;[12] RA1, AN1 or C12IN0- (CIS=0), ICSP Clock
GAINSEL		EQU		2			;[11] RA2, AN2 or C1OUT, 
VPP			EQU		3			;[4], RA3, DIO3, MCLR, ICSP VPP
VCAP		EQU		4			;[3], RA4, AN3, DIO4, 
SPARE		EQU		5			;[2], RA5, DIO5 (Can be used for status indicator)
AD_Vcap		EQU		ANS3		;(4), voltage on capacitor

; ***** PORTC definitions *****
AD_Vout		equ		ANS4		;[10]RC0, AN4, C2IN+  
AD_Vpot		equ		ANS5		;(9) RC1, AN5 or C12IN1- Use for 10k pot for temp control
AD_Iout		equ		ANS6		;[8] RC2, AN6, C12IN2- 
AD_Vin		equ		ANS7		;[7] RC3, AN7, C12IN3- = Input voltage/6: 12 volts = 255*12/(5*6) = 102
THERM		equ		4			;[6] RC4, DIO4, C2 Out Jumper to pin 9
CCP1		equ		5			;[5] RC5, DOUT5, PWM output

;***** VARIABLE DEFINITIONS *****
	UDATA	0x20
tm0count	RES	1			;Increments 250 uSec or 4 KHz
tm0count1	res	1			;Increments 64 mSec or 18 Hz
Vin			RES	1			;Voltage on Input = AN4
Vpot		res	1			;Voltage from pot
ADCnum		RES	1			;ADC number to select: 0, 4, 6, 7
PWMvalue	RES	1			;PWM value 0-48, 0-96, or 0-192 for 0-96% AT 200/100/50 KHz
PWMtemp		res	1			;Temporary PWM value for setting CCP
ADCstat		res	1			;Set bits when ADC read, reset when action taken
DelayCount	res	1			;Delay in 250 mSec increments, 4 = 1 sec
PortAdata	res	1
TblPtr		res	1			;Table Pointer
HighPower	res	1			;Non zero is True
StartDelay	res	1			;Start Delay - decrement in ComputePWM
	UDATA_SHR	0x71			;Dual address overlapped with 0xf1-0xff
w_temp		RES	1			; variable used for context saving
status_temp	RES	1			; variable used for context saving
EEPROMaddr	res	1			;Address in EEPROM to read/write
EEPROMdata	res	1			;Data in EEPROM to read/write

;**********************************************************************
	ORG		0x000			; processor reset vector
	nop						;Req'd by debugger?
  	goto	Main			; go to beginning of program


	ORG		0x004			; interrupt vector location
	goto		Isr

;**********************************************************************
	CODE
Isr:
	movwf		w_temp			; save off current W register contents
	movf		STATUS,w		; move status register into W register
	movwf		status_temp		; save off contents of STATUS register
	bcf			INTCON,GIE		;Turn off Global Interrupts

Timer0InterruptCheck
	BANK0							;*** Need this if INT occurs in BANK1
	btfsc	INTCON,T0IF				;Bank 0
	goto	Timer0Interrupt			;Yes
	
ADIFInterruptCheck
	btfsc	PIR1,ADIF				;Bank 0
	goto	A_DConverterInterrupt	;Yes

Timer0Interrupt
	movlw	d'164'				;Set up for 100 uSec (256-164 = 92), 50 uSec (256-212 = 42)
	movwf	TMR0
	bsf		ADCON0, GO			;Bank 0, START Next conversion
	incf	tm0count,f			;Rolls over 100*256 uSec = 25.6 mSec = 39 Hz
	btfsc	STATUS, Z

	incf	tm0count1,F			;Increments at 39 Hz

EndTimer0Interrupt
	bcf		INTCON,T0IF			;Bank 0, Clear TMR0 Interrupt Flag
	goto	EndIsr

;When A/D converter has a result, store in proper register and select next
; 01/12/08 - Load PRT for 100 uSec A/D, software filter Iout

A_DConverterInterrupt			;44 uSec conversion (was 51 uSec?)
	bcf		PIR1, ADIF			;Bank 0

ADCint10:						
	movf	ADCnum,W
	sublw	AD_Vin
	bnz		ADCint40			;Go to next A/D

;Process AD_Vin = AD7	
	bsf		ADCstat, AD_Vin		;Indicate reading
	movf	ADRESH,W			;Bank 0
	movwf	Vin					;0-255 corresponds to 0-30V
	movlw	AD_Vpot
	movwf	ADCnum				;Next is AD5 = AD_Vpot	
	movlw	SEL_AD5
	movwf	ADCON0				;Bank 0	
	goto	EndIsr

ADCint40:
	movf	ADCnum,W
	sublw	AD_Vpot
	bnz		EndIsr				;Go to next A/D

;Process AD_Vpot = AD5	
	bsf		ADCstat, AD_Vpot	;Indicate reading
	movf	ADRESH,W			;Bank 0
;	movlw	d'50'
	movwf	Vpot				;0-255 corresponds to 0-5V
	movlw	AD_Vin
	movwf	ADCnum				;Read Vpot, then Next is AD7 = AD_Vin	
	movlw	SEL_AD7
	movwf	ADCON0				;Bank 0	

EndIsr
	movf		status_temp,w	; retrieve copy of STATUS register
	movwf		STATUS			; restore pre-isr STATUS register contents
	swapf		w_temp,f
	swapf		w_temp,w		; restore pre-isr W register contents
	retfie						; return from interrupt, enable GIE

;********************************* Start of Program ********************************
Main
	BANK1
	clrf	ANSEL				;Bank 1, Set all to digital
	bsf		ANSEL,ANS5			;AN5 start as ADC

;	movlw	b'00010001'	;fOsc/8 = 8MHz/8 = 1 uSec * 11 = 11 uSec (test 51)
;	movlw	b'01010001'	;fOsc/16 = 8MHz/16 = 2 uSec * 11 = 22 uSec (test 43)
	movlw	b'00100000'	;fOsc/32 = 8MHz/32 = 4 uSec * 11 = 44 uSec (sim test 50)
;	movlw	b'01100001'	;fOsc/64 = 8MHz/64 = 8 uSec * 11 = 88 uSec (test ERR)
	movwf	ADCON1		;Bank 1
	bsf		PIE1,ADIE	;Bank 1, Enable ADC interrupt
	BANK0

;ADCON0 <7> Just, <6> Ref, <5:2> CHS, <1> GO/Done, <0> ADON
	movlw	SEL_AD5		;Left justified, Vdd Vref, Select AN5, ON b'00001101'
	movwf	ADCON0		;Bank 0

;PWM setup
	movlw	b'00001111'	;Active LOW output (High turns MOSFET off)
	movwf	CCP1CON		;Bank 0, Set up compatible PWM on output RC5

	BANK1
	movlw	b'11011011'	;RA2, RA5 output, RA4 (Vcap) input
	movwf	TRISA
	movlw	b'11111111'	;RC5(PWM) Outputs (Disable)
	movwf	TRISC		;Tri-State All Inputs

; Timer 0 source internal, prescale 2:1 = 1000 KHz/256 = 3904 Hz				
	movlw	b'10000000'	
	movwf	OPTION_REG	;Bank 1, TIMER0 Prescaler <0:2>, disable port A pullups <7>

;	movlw	d'63'		;For 256 counts = 100%, F = 8/(64*4*4) = 7.81 kHz
	movlw	d'41'		;For 168 counts = 100% (3.333V = 171) and 8/(42*4*4) = 11.90 kHz
	movwf	PR2			;Bank 1
; PWM 100% determined by (PR2+1) * PRE counts

; PWM period determined by: (PR2+1) * 4 * Tosc * TMR2prescale
; 100 kHz is 10 uSec. for 8 MHz clock, PRT2 = 19 with 1:1 prescale 80 counts = 100%
	BANK0
	if	PWMFREQ == 3
	movlw	b'00000010'		;Prescale 16
	endif
	if PWMFREQ == 12
	movlw	b'00000001'		;Prescale 4
	endif
	if PWMFREQ == 48
	movlw	b'00000000'		;Prescale 1
	endif
	movwf	T2CON
	bsf		T2CON,TMR2ON	;Bank 0 - turn timer on

	clrf	PORTC

	bsf		INTCON,PEIE	;Bank 0, Enable peripheral interrupts
	bsf		INTCON,GIE 	;Interrupt on Change Enabled
	bsf		INTCON,T0IE ;Interrupt on TIMER0 Overflow Enabled
	bcf		INTCON,T0IF ;Clear TIMER0 Overflow Interrupt Flag

;Duty Cycle (time) is determined by: CCP * Tosc * TMR2prescale
;5 uSec (50%) is 20 for 8 MHz clock 1:1 prescale
;At 100 kHz, for CCP = 63, tOn = 63*1/8 = 7.875/10 = 78% (80 = 100%)
	movlw	d'10'
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
	bcf		PIR1, TMR2IF	;Bank 0

	movlw	AD_Vin
	movwf	ADCnum
	bcf		ADCstat,AD_Vin
	bsf		ADCON0, GO		;Bank 0, start conversion
	bsf		INTCON,GIE		;Turn on Global Interrupts

	BANK1
	bsf		ANSEL,AD_Vpot		;AN5
	bsf		ANSEL,AD_Vin		;AN7
	BANK0

	movlw	AD_Vin
	movwf	ADCnum			;Start with AD_Vin	
	clrf	ADCstat			;clear ready flags
	movlw	d'100'
	movwf	StartDelay
	clrf	TMR0			;Bank 0

;Wait for voltages to settle
	movlw	d'5'			;5 * 50 = 250 mSec
	movwf	DelayCount
	call	Delay

	call	EnablePWM

; *********************************** Main Program Loop ****************************
Loop:
	clrwdt

;***********************************************************************************
CheckADC:

;***********************************************************************************
CheckVpot:
	btfss	ADCstat, AD_Vpot	;ANS5
	goto	CheckVin			;wait until AD_Vout read

;Read Vpot, Vin every 200 uSec
ReadVpot:
	bcf		ADCstat, AD_Vpot
	movf	Vpot,W
	movwf	PWMvalue
	call	SetPWM
	goto	EndLoop

;***********************************************************************************
CheckVin:
	btfss	ADCstat, AD_Vin		;ANS7
	goto	EndLoop				;wait until AD_Vin read

ReadVin:
	bcf		ADCstat, AD_Vin
	movf	Vin,W
	sublw	MINVIN				;W = MINVIN-Vin
	btfss	STATUS,C
	goto	CheckVin2			;Borrow: Vin >= MINVIN

VinTooLow:
	call	ShutDownPWM
	goto	EndLoop

CheckVin2:						;Vin >= MINVIN
	movf	Vin,W
	sublw	MAXVIN				;W = MAXVIN-Vin
	btfsc	STATUS,C
	goto	VinOK				;No Borrow: Vin <= MAXVIN

VinTooHigh:						;Vin > MAXVIN
	call	ShutDownPWM
	goto	EndLoop

VinOK:
	call	EnablePWM

EndLoop:
	goto	Loop


; *********************** Adjust PWM to value in PWMval **********************
; Duty Cycle ratio is (CCPR1L + CCP1CON<4:5>)/(4*(PR2+1)
; PR2 = 65, Max PWM = 1024/264
; 

SetPWM:
	movf	PWMvalue,W
	movwf	PWMtemp
	bcf		CCP1CON, 5			;Bank 0
	bcf		CCP1CON, 4
	btfsc	PWMtemp, 0
	bsf		CCP1CON, 4
	btfsc	PWMtemp, 1
	bsf		CCP1CON, 5			;Set LSBs

	rrf		PWMtemp,F
	rrf		PWMtemp,F			;Adjust to MSBs
	movf	PWMtemp,W			;move to WREG
	andlw	0x3f
	movwf	CCPR1L				;Bank 0, Set MSBs for PWM

	return

; *********************** Turn PWM Output OFF and Latch ***************
ShutDownPWM:
	BANK1						;No Borrow: Vout > MAXVOUT
	bsf		TRISC, CCP1			;Shut down PWM by tristating output pin
	BANK0
	return

; *********************** Turn PWM Output OFF *************************
DisablePWM:
	BANK1						;No Borrow: Vout > MAXVOUT
	bsf		TRISC, CCP1			;Shut down PWM by tristating output pin
	BANK0
	return

; *********************** Turn PWM Output ON **************************
EnablePWM:
	BANK1						;No Borrow: Vout > MAXVOUT
	bcf		TRISC, CCP1			;Enable PWM by un-tristating output pin
	BANK0
	return

; ***************** Delay 50 mSec * DelayCount *************************
Delay:
	movlw	d'60'				;255-60 = 195 counts at 256 uSec = 50 mSec
	movwf	tm0count
Delay10:			;
	clrwdt
	movf	tm0count,W
	iorwf	tm0count,W
	btfss	STATUS,Z
	goto	Delay10

	decf	DelayCount, F
	btfss	STATUS,Z
	goto	Delay

	return						;return when delay = 0


;********************** ComputePWM *****************************
;Use LEDcount and Vin to determine PWMvalue

ComputePWM:
	movf	StartDelay,W
	iorwf	StartDelay,W
	btfsc	STATUS,Z
	goto	Compute				;If result is zero

	incf	StartDelay,F		;Wait for start delay before computing
	return

Compute:
	clrf	TblPtr
	movf	Vin,W
	sublw	d'111'				;13 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow (CF clear): Vin > Vset

	incf	TblPtr,F
	movf	Vin,W
	sublw	d'107'				;12.5 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow: Vin > Vset

	incf	TblPtr,F
	movf	Vin,W
	sublw	d'102'				;12 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow: Vin > Vset

	incf	TblPtr,F
	movf	Vin,W
	sublw	d'98'				;11.5 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow: Vin > Vset

	incf	TblPtr,F
	movf	Vin,W
	sublw	d'94'				;11 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow: Vin > Vset

	incf	TblPtr,F
	movf	Vin,W
	sublw	d'89'				;10.5 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow: Vin > Vset

	incf	TblPtr,F
	movf	Vin,W
	sublw	d'85'				;10 VDC
	btfss	STATUS,C
	goto	Lookup7				;Borrow: Vin > Vset

	incf	TblPtr,F

Lookup7:
	movlw 	LOW Tbl7 	;get low 8 bits of address
	incf	TblPtr,F		;Do increment first, becaus it doesn't affect the carry flag.
								;	The increment takes into account that the first thing in Table
								;	is not data, but an instruction
	addwf 	TblPtr,F 	;do an 8-bit add operation
	movlw 	HIGH Tbl7 	;get high 5 bits of address
	btfsc 	STATUS,C 			;page crossed?
	addlw 	1 					;yes then increment high address
	movwf 	PCLATH 				;load high address in latch
	movf 	TblPtr,w 	;load computed offset in w reg
	call 	Tbl7
	movwf	PWMvalue

	return


Tbl7:
	movwf 	PCL 	;load computed offset in PCL
;	retlw 	D'20' 	;
	retlw 	D'23' 	;13
	retlw 	D'24' 	;12.5
	retlw 	D'25' 	;12
	retlw 	D'25' 	;
	retlw 	D'25' 	;
	retlw 	D'26' 	;
	retlw 	D'26' 	;
	retlw 	D'27' 	;
	retlw 	D'27' 	;
	retlw 	D'28' 	;
	retlw 	D'28' 	;

	retlw	d'15'	;just in case

;**********************************************************************
; Comparator details
; C12IN1- is pin 9 connected to MOSFET CURRENT
; C1INREF is COMP1 + input
; Set C1ON to logic 1
; Set C1POL for proper polarity for PWM logic
; Overcurrent on MOSFET trips C1 and sets C1IF for interrupt
; CM1CON0
; Bit 7 C1ON = 1 to enable Comparator 1
; Bit 6 C1OUT = 0 when C1+ > C1- (Inverted polarity C1POL = 1)
; Bit 5 C1OE = 0 for internal output connection only
; Bit 4 C1POL = 1 for inverted logic
; Bit 3 
; Bit 2 C1R = 1 for reference connected to C1REF
; Bits 0:1 C1CH = 01 for C12IN1 as inverting input
; CM2CON0
; Bit 7 C2ON = 0 to disable Comparator 1
; Bit 5 C1OE = 0 for internal output connection only
; VRCON
; Bit 1 C1VREN = 1 for reference circuit C1; = 0 for 600 mV fixed reference C1
; Bit 2 C2VREN = 1 for reference circuit C2; = 0 for 600 mV fixed reference C2
; Bit 5 VRR = 1 for low range, = 0 for high range
; Bit 4 FVREN = 1 for fixed voltage 0.6 V enabled (0 disabled)
; Bits 3:0  Low range: CVREF = (VR<3:0>/24) * VDD or 0 to (15/24)*5 = 3.125
;			High range: CVREF = VDD/4 + (VR<3:0>/32) * VDD or 1.25 to 3.594

; PWM shutdown
; ECCPAS
; Bit 7 ECCPASE = 1 to indicate shutdown event occurred
; Bits 6:4 ECCPAS = 001 for C1 output high for shutdown
; Bits 3:2 PSSACn = 1x P1A and P1C tristate
; Bits 0:1 PSSBDn = 1x P1B and P1D tristate



	END                       ; directive 'end of program'

