 ;Timer0.asm 
;Reads ADC (AN0, pin 7) and sets a delay accordingly. 
 
    #include    "P12F675.INC" 
 
    #define LED 2    ;GP2 (pin 5) 
    #define INIT_COUNT 10    ; 
 
    errorlevel -302        ;suppress "not in bank 0" message 
 
    cblock    0x20 
    tick_counter 
    temp 
    ADC_value_lo 
    ADC_value_hi     
    endc 
 
    ;reset vector 
    org        0 
    clrf    PCLATH 
    goto    main 
 
    ;interrupt vector 
    org        4 
    banksel    INTCON 
    bcf        INTCON,T0IF        ;clear Timer0 interrupt flag 
    movlw    INIT_COUNT        ;re-initialise count 
    movwf    TMR0 
    decf    tick_counter,f 
    retfie 
 
main: 
    banksel TRISIO 
    movlw    0x00            ;all pins outputs 
    movwf    TRISIO 
    bsf        TRISIO, GP0        ;AN0 input 
    banksel ANSEL 
    movlw     0x01             ;AN0 as analog input 
    movwf     ANSEL 
    banksel    ADCON0 
    movlw    0x01            ;ADC clock Fosc/8, channel 0, ADC turned on (01000001) 
    movwf     ADCON0 
    banksel    OPTION_REG 
    movlw    b'00000011'        ;prescaler 1/128 
    movwf    OPTION_REG        ; 
    banksel    TMR0 
    movlw    INIT_COUNT        ;initialise timer count value 
    clrf    tick_counter 
    movwf    TMR0 
    bsf        INTCON,GIE        ;enable global interrupt 
    bsf        INTCON,T0IE        ;enable Timer0 interrupt 
    banksel    0 
 
mainloop: 
    call    read_ADC 
    bsf        GPIO,LED 
    call    dly 
    bcf        GPIO,LED 
    call    dly     
    goto    mainloop 
 
dly: 
    movfw    ADC_value_hi 
    movwf    tick_counter 
dly1: 
    movf    tick_counter,f 
    skpz 
    goto    dly1 
    return 
 
;--------------------------------------------------------------------- 
;read_ADC 
;function to read ADC 
;returns high and low bytes (10 bits) in ADC_value_hi 
;and ADC_value_lo variables. Value is left-justified. 
;--------------------------------------------------------------------- 
 
read_ADC: 
    banksel ADCON0 
    bsf     ADCON0,GO    ;start conversion 
WaitADC: 
    btfss     ADCON0,GO    ;GO/DONE clear when conversion is complete 
    goto     WaitADC        ;wait for ADC to finish 
GetADC1: 
    banksel    ADRESH 
    movfw     ADRESH        ;get high byte in W 
    movwf     ADC_value_hi 
    bcf        STATUS,Z 
    rrf        ADC_value_hi,f    ;divide by 2 
    movlw    .10                ;and add a constant 
    addwf    ADC_value_hi,f    ;to give a narrow range of values 
    return 
 
    end 
