;**********************************************************************
;                                                                     *
;    Filename:	    PWMRLY.asm                                       *
;    Date:          07/18/10                                          *
;    File Version:  00                                                *
;    Checksum:		0x                                                * 
;                                                                     *
;    Author:        Paul E. Schoen                                    *
;    Company:       P S Technology, Inc.                              *
;                                                                     *
;**********************************************************************
;    Notes:                                                           *
; 07/18/10 -  - Create from FL7-13H
; This application is designed to power a 24 VAC relay coil from 24 VDC
; On startup, turn MOSFET off, and wait for voltage to be > 22 VDC
; Turn MOSFET on full (PWM = 100%) for 50 mSec
; Adjust PWM to 10% and maintain until voltage is removed
; 
;**********************************************************************
; PWM inhibited if Vin < MINVIN

	list		p=16HV616		; list directive to define processor
	#include	<P16HV616.inc>		; processor specific variable definitions
	
	__CONFIG    _CP_OFF & _BOD_ON & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_ON
;	__CONFIG    _CP_OFF & _CPD_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _FCMEN_OFF & _IESO_OFF

#define	BANK1		banksel 0x80	;Select BANK1, STATUS<5> = 1
#define	BANK0		banksel 0x00	;Select BANK0, STATUS<5> = 0


;PWM frequency may be 2.98 (3), 11.90 (12), or 47.62 kHz (48)
#define		PWMFREQ		3

; Actual A/D values for measured inputs

; VIN: 8.53 counts per volt: 22V = 188, 26 = 222
#define		MINVIN		D'188'	;Actual Vin * MaxADC(256) / Vref(5) / Ratio(6)
#define		MAXVIN		D'222'	;Actual Vin * MaxADC(256) / Vref(5) / Ratio(6)

;Bit 7=0 for left justified, bit 6 = 0 for Vdd ref
#define		SEL_AD1		b'00000101'				;001 = AD1
#define		SEL_AD3		b'00001101'				;011 = AD3
#define		SEL_AD4		b'00010001'				;100 = AD4
#define		SEL_AD5		b'00010101'				;101 = AD5
#define		SEL_AD6		b'00011001'				;110 = AD6
#define		SEL_AD7		b'00011101'				;111 = AD7

; ***** PORTC definitions *****
AD_Vin		equ		ANS7		;[7] RC3, AN7, C12IN3- = Input voltage/6: 12 volts = 255*12/(5*6) = 102
THERM		equ		4			;[6] RC4 Input, Jumpered to CCP1, THERM used as pull-down
CCP1		equ		5			;[5] RC5, DOUT5, PWM output to MOSFET gate

;***** VARIABLE DEFINITIONS *****
	UDATA	0x20
tm0count	RES	1			;Increments 250 uSec or 4 KHz
tm0count1	res	1			;Increments 64 mSec or 18 Hz
Vin			RES	1			;Voltage on Input = AN7
PWMvalue	RES	1			;PWM value 0-168 
DelayCtr	res	1
DelayCount	res	1			;Delay in 10 mSec increments, 4 = 1 sec

	UDATA_SHR	0x71			;Dual address overlapped with 0xf1-0xff
w_temp		RES	1			; variable used for context saving
status_temp	RES	1			; variable used for context saving

;**********************************************************************
	ORG		0x000			; processor reset vector
	nop						;Req'd by debugger?
  	goto	Main			; go to beginning of program


	ORG		0x004			; interrupt vector location
	goto		Isr

;**********************************************************************
	CODE
Isr:
	movwf		w_temp			; save off current W register contents
	movf		STATUS,w		; move status register into W register
	movwf		status_temp		; save off contents of STATUS register
	bcf			INTCON,GIE		;Turn off Global Interrupts

Timer0InterruptCheck
	BANK0							;*** Need this if INT occurs in BANK1
	btfsc	INTCON,T0IF				;Bank 0
	goto	Timer0Interrupt			;Yes
	
ADIFInterruptCheck
	btfsc	PIR1,ADIF				;Bank 0
	goto	A_DConverterInterrupt	;Yes

Timer0Interrupt
	movlw	d'164'				;Set up for 100 uSec (256-164 = 92), 50 uSec (256-212 = 42)
	movwf	TMR0
	incf	DelayCtr,f			;For time delay
	incf	tm0count,f			;Rolls over 100*256 uSec = 25.6 mSec = 39 Hz
	btfss	STATUS, Z
	goto	EndTimer0Interrupt

	incf	tm0count1,F			;Increments at 25.6 mSec 39 Hz
	bsf		ADCON0, GO			;Bank 0, START Next conversion 39/sec

EndTimer0Interrupt
	bcf		INTCON,T0IF			;Bank 0, Clear TMR0 Interrupt Flag
	goto	EndIsr

;When A/D converter has a result, store in proper register and select next
A_DConverterInterrupt			;100 uSec conversion
	bcf		PIR1, ADIF			;Clear Interrupt Flag; Bank 0

;Process AD_Vin = AD7	
	movf	ADRESH,W			;Bank 0
	movwf	Vin					;0-255 corresponds to 0-30V

EndIsr
	movf		status_temp,w	; retrieve copy of STATUS register
	movwf		STATUS			; restore pre-isr STATUS register contents
	swapf		w_temp,f
	swapf		w_temp,w		; restore pre-isr W register contents
	retfie						; return from interrupt, enable GIE

;********************************* Start of Program ********************************
Main
	BANK1
	clrf	ANSEL				;Bank 1, Set all to digital
	bsf		ANSEL,ANS5			;AN5 start as ADC

	movlw	b'00100000'	;fOsc/32 = 8MHz/32 = 4 uSec * 11 = 44 uSec (sim test 50)
	movwf	ADCON1		;Bank 1
	bsf		PIE1,ADIE	;Bank 1, Enable ADC interrupt
	BANK0

;ADCON0 <7> Just, <6> Ref, <5:2> CHS, <1> GO/Done, <0> ADON
	movlw	SEL_AD7		;Left justified, Vdd Vref, Select AN7, ON b'00011101'
	movwf	ADCON0		;Bank 0

;------------------------------ PWM setup --------------------------------------
	movlw	b'00001100'	;Active HIGH output (High turns MOSFET ON)
	movwf	CCP1CON		;Bank 0, Set up compatible PWM on output RC5

	BANK1
	movlw	b'11000000'	;RA0:5 output
	movwf	TRISA
	movlw	b'11111111'	;RC5(PWM) Outputs (Disable)
	movwf	TRISC		;Tri-State All Inputs

; Timer 0 source internal, prescale 2:1 = 1000 KHz/256 = 3904 Hz				
	movlw	b'10000000'	
	movwf	OPTION_REG	;Bank 1, TIMER0 Prescaler <0:2>, disable port A pullups <7>

; PWM 100% determined by (PR2+1) * PRE counts
; PWM period determined by: (PR2+1) * 4 * TMR2prescale / Clock
; PR2=167 => 336 uSec
	movlw	d'167'
	movwf	PR2			;Bank 1

	BANK0
	if PWMFREQ == 3
	movlw	b'00000001'		;Prescale 4
	endif
	if PWMFREQ == 12
	movlw	b'00000000'		;Prescale 1
	endif
	movwf	T2CON
	bsf		T2CON,TMR2ON	;Bank 0 - turn timer on

	clrf	PORTC

	bsf		INTCON,PEIE	;Bank 0, Enable peripheral interrupts
	bcf		INTCON,GIE 	;Interrupt on Change DISabled
	bsf		INTCON,T0IE ;Interrupt on TIMER0 Overflow Enabled
	bcf		INTCON,T0IF ;Clear TIMER0 Overflow Interrupt Flag

;Duty Cycle (time) is determined by: CCP * Tosc * TMR2prescale
; 2.98 kHz => 336 uSec
; Clock is 8 MHz
;At 3 kHz, for CCP(MSB) = 167, tOn = 167*4*4/8 = 334 uSec (100%)
;Duty Cycle = CCPR / (4 * (PR2+1)) = CCPR / 672 = CCPR(MSB)/168
; Keeping LSBs = 0, an MSB of 168 is 100%, and 17 is about 10%
	movlw	d'168'			;Set Duty Cycle 100%
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
	bcf		PIR1, TMR2IF	;Bank 0

	bsf		ADCON0, GO		;Bank 0, start conversion
	bsf		INTCON,GIE		;Turn on Global Interrupts

	clrf	TMR0			;Bank 0

;Wait for voltages to settle
	movlw	d'10'			;10 * 10 = 100 mSec
	movwf	DelayCount
	call	Delay10

; *********************************** Main Program Loop ****************************
Loop:

;*********************** Read Vin and loop until voltage OK **************************
ReadVin:
	clrwdt
	movf	Vin,W
	sublw	MINVIN				;W = MINVIN-Vin
	btfss	STATUS,C
	goto	CheckVin2			;Borrow: Vin >= MINVIN

VinTooLow:
	call	ShutDownPWM
	goto	ReadVin				;Loop with PWM off

CheckVin2:						;Vin >= MINVIN
	movf	Vin,W
	sublw	MAXVIN				;W = MAXVIN-Vin
	btfsc	STATUS,C
	goto	VinOK				;No Borrow: Vin <= MAXVIN

VinTooHigh:						;Vin > MAXVIN
	call	ShutDownPWM
	goto	ReadVin

VinOK:
	movlw	D'168'				;Set to 100%
	movwf	PWMvalue
	call	SetPWM
	call	EnablePWM

PWM100:
	movlw	d'10'			;10 * 5 = 50 mSec
	movwf	DelayCount
	call	Delay10

PWM10:
	clrwdt
	movlw	D'16'				;Set to 10%
	movwf	PWMvalue
	call	SetPWM

ReadVin10:
	clrwdt
	movf	Vin,W
	sublw	MINVIN				;W = MINVIN-Vin
	btfss	STATUS,C
	goto	ReadVin10			;Borrow: Vin >= MINVIN

VinTooLow10:
	call	ShutDownPWM

EndLoop:
	goto	ReadVin				;Wait for voltage to come back up


; *********************** Adjust PWM to value in PWMval **********************
; Use MSB only, so 100% is 168
; Duty Cycle ratio is (CCPR1L + CCP1CON<4:5>)/(4*(PR2+1)
; PR2 = 65, Max PWM = 1024/264
; 

SetPWM:
	movf	PWMvalue,W
	movwf	CCPR1L
	bcf		CCP1CON, 5			;Set LSBs to 0; Bank 0
	bcf		CCP1CON, 4
	return

; *********************** Turn PWM Output OFF and Latch ***************
ShutDownPWM:
	BANK1						;No Borrow: Vout > MAXVOUT
	bsf		TRISC, CCP1			;Shut down PWM by tristating output pin
	BANK0
	return

; *********************** Turn PWM Output OFF *************************
DisablePWM:
	BANK1						;No Borrow: Vout > MAXVOUT
	bsf		TRISC, CCP1			;Shut down PWM by tristating output pin
	BANK0
	return

; *********************** Turn PWM Output ON **************************
EnablePWM:
	BANK1						;No Borrow: Vout > MAXVOUT
	bcf		TRISC, CCP1			;Enable PWM by un-tristating output pin
	BANK0
	return

; ***************** Delay 10 mSec * DelayCount *************************
Delay10:
	movlw	d'155'				;255-155 = 100 counts at 100 uSec = 10 mSec
	movwf	DelayCtr
Delay10a:			;
	clrwdt
	movf	DelayCtr,W
	iorwf	DelayCtr,W
	btfss	STATUS,Z
	goto	Delay10a

	decf	DelayCount, F
	btfss	STATUS,Z
	goto	Delay10

	return						;return when delay = 0

	END                       ; directive 'end of program'

