;**********************************************************************
;                                                                     *
;    Filename:	    PWMRLY_12F675.asm                                       *
;    Date:          07/18/10                                          *
;    File Version:  00                                                *
;    Checksum:		0x                                                * 
;                                                                     *
;    Author:        Paul E. Schoen                                    *
;    Company:       P S Technology, Inc.                              *
;                                                                     *
;**********************************************************************
;    Notes:                                                           *
; 07/18/10 -  - Create from FL7-13H
; This application is designed to power a 24 VAC relay coil from 24 VDC
; On startup, turn MOSFET off, and wait for voltage to be > 22 VDC
; Turn MOSFET on full (PWM = 100%) for 50 mSec
; Adjust PWM to 10% and maintain until voltage is removed
; 
;**********************************************************************
; PWM inhibited if Vin < MINVIN

	list		p=12F675		; list directive to define processor
	#include	<P12F675.inc>		; processor specific variable definitions
	
	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_ON & _MCLRE_ON & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT  
;	__CONFIG    _CP_OFF & _BOD_ON & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_ON
;	__CONFIG    _CP_OFF & _CPD_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _FCMEN_OFF & _IESO_OFF

#define	BANK1		banksel 0x80	;Select BANK1, STATUS<5> = 1
#define	BANK0		banksel 0x00	;Select BANK0, STATUS<5> = 0

#define	VIN		ANS2				;GP4 Pin 3
#define	PWMOUT	GP5					;GP2 Pin 2

; <6:4> ClkSel 001=fOsc/8, <3:0> ANSEL
#define	ANSELSET	b'00000100'		;Used to configure AD ANS2

;Bit 7=0 for left justified, bit 6 = 0 for Vdd ref, <3:2> CHsel, 1=GO/DONE, 0=ADON
; #define	ADCON0SET	b'10001101'		;Channel ANS3
#define	ADCON0SET	b'10001001'		;Channel ANS2

; <7>GPPU 1=disable, <6>INTEDG, <5>T0CS 0=Osc, <4>T0SE 0=L-H, <3>PSA 0=TMR0,1=WDT, <2:0>PS 000=1/2, 001=1/4
#define	OPTIONSET	b'10000001'		;TMR0 4 uSec

#define	TRISIOSET	b'00110100'		;Set 2,4,5 as input, <0,1,3> as output

; Actual A/D values for measured inputs

; VIN: 8.53 counts per volt: 22V = 188, 26 = 222
; 24V = 820 counts, 6 samples, divide by 32 = 154
#define	MINVIN		D'128'	;D'128'	;20V Actual Vin * MaxADC(1024) / Vref(5) / Ratio(6)
#define	MAXVIN		D'166'	;26V Actual Vin * MaxADC(1024) / Vref(5) / Ratio(6)

#define MAXSAMPLE	.12			;Maximum sample for averaging( 800 uSec*6 = 4.8 mSec )

#define	THRESH1		d'120'		;180V
#define	THRESH2		d'100'		;145V

#define	COMP		d'5'		;Compensation for ISR overhead

;***** VARIABLE DEFINITIONS *****
; Note: All RAM data is shared
	udata_shr
tm0count	res	1			;Increments 250 uSec or 4 KHz
tm0count1	res	1			;Increments 64 mSec or 18 Hz
Vin			res	1			;Voltage on Input = AN7
PWMvalue	res	1			;PWM value 0-168 
DelayCtr	res	1
DelayCount	res	1			;Delay in 10 mSec increments, 4 = 1 sec
ADCRES		res	1			; ADC result
AVGADC		res	1			; Average ADC value 0-182
AVGLO		res	1			; Average ADC value low byte
AVGHI		res	1			; Average ADC value high byte
SAMPLE		res	1			; Sample number decrement from MAXSAMPLE to zero
OUTREG		res	1			; Output register
AVGCTR		res	1			; ISR counter
TM0CTR		res	1			; Timer0 Counter
OnTime		res	1			; PWM ON time in uSec 0-255
OffTime		res	1			; PWM OFF time in uSec 0-255

WTEMP		res 1			; register used in Interrupt Routine
STATUSTEMP	res 1			; register used in Interrupt Routine
PCLATHTEMP	res 1			; register used in Interrupt Routine
FSRTEMP		res 1			; register used in Interrupt Routine
FLAGS		res 1			; register used to set flags
	CODE
;**********************************************************************
	ORG		0x000			; processor reset vector
	nop						;Req'd by debugger?
  	goto	Main			; go to beginning of program


	ORG		0x004			; interrupt vector location
	goto		Isr

;**********************************************************************
Isr:
	movwf   WTEMP         	;Save off current W register contents
	movf	STATUS,w
	movwf	STATUSTEMP									 
	movf	PCLATH,w
	movwf	PCLATHTEMP		;Save PCLATH
	movf	FSR,w
	movwf	FSRTEMP			;Save FSR
	BANKSEL	0	  

;****************************************************************************
;Interrupt Source Checks
;****************************************************************************
Timer0InterruptCheck
	btfsc	INTCON,T0IF		;Is TOIF Set?
	goto	Timer0Interrupt	;Yes
	
ADCInterruptCheck
	btfsc	PIR1,ADIF		;Is ADIF Set?
	goto	ADCInterrupt	;Yes	
	
	goto	EndIsr			;No

Timer0Interrupt:			;4 uSec * 200 = 800 iSec
	btfsc	GPIO,PWMOUT		;16 instruction overhead
	goto	PWMoff

PWMon:
	movf	OnTime,W
	movwf	TMR0
	bsf		GPIO,PWMOUT
	goto	EndPWM

PWMoff:
	movf	OffTime,W
	movwf	TMR0
	bcf		GPIO,PWMOUT

EndPWM:
	incf	TM0CTR, F		;Increment counter
	bcf		INTCON,T0IF		;Clear TMR0 Interrupt Flag

	btfsc	GPIO,PWMOUT		;Process delay only on off times
	goto	EndTimer0Interrupt

	bsf		ADCON0, GO			;Bank 0, START Next conversion 800 uSec

	movf	DelayCtr,W
	iorwf	DelayCtr,W
	bz		DelayZero
	decf	DelayCtr,f			;For time delay
DelayZero:
	incf	tm0count,f			;Rolls over 100*256 uSec = 204 mSec
	btfss	STATUS, Z
	goto	EndTimer0Interrupt

	incf	tm0count1,F			;Increments at 256*800uSec=

EndTimer0Interrupt
	bcf		INTCON,T0IF			;Bank 0, Clear TMR0 Interrupt Flag
	goto	EndIsr

;When A/D converter has a result, store in proper register and select next
ADCInterrupt
	BANKSEL	ADRESL
	movf	ADRESL,W 		;Move the least significant byte of A/D Result to W
	BANKSEL	0
	addwf	AVGLO, F		;Add and store in AVGLO
	btfss	STATUS, C		;If carry flag is set, increment AVGHI
	goto	AVGNC			;else continue

	incf	AVGHI, F		;Increment AVGHI and store

AVGNC
	movf	ADRESH,W
	addwf	AVGHI, F

	decf	SAMPLE, F		;Decrement sample counter
	btfss	STATUS, Z		;If Zero flag is set, end sample and update AVGADC
	goto	EndADCInterrupt
	
	bcf		STATUS, C		;Reset carry flag
	rrf		AVGHI, F		; 6 samples at 24V (820) => 4920
	rrf		AVGLO, F		;/2
	rrf		AVGHI, F
	rrf		AVGLO, F		;/4
	rrf		AVGHI, F
	rrf		AVGLO, F		;/8
	rrf		AVGHI, F
	rrf		AVGLO, F		;/16
	rrf		AVGHI, F
	rrf		AVGLO, F		; Divide by 32 = 154
	rrf		AVGHI, F
	rrf		AVGLO, F		; Divide by 32 = 154
	movf	AVGLO, W
	movwf	AVGADC
	movlw	MAXSAMPLE
	movwf	SAMPLE
	clrf	AVGHI
	clrf	AVGLO
;	movf	OUTREG,W
;	xorlw	b'00000100'		;Toggle G2 pin 5
;	movwf	OUTREG
;	movwf	GPIO
EndADCInterrupt:
	bcf		PIR1, ADIF		;ADC conversion 100 uSec
	
EndIsr
	movf	FSRTEMP,w
	movwf	FSR				  ;Restore FSR
	movf	PCLATHTEMP,w
	movwf	PCLATH			  ;Restore PCLATH
	movf	STATUSTEMP,w
	movwf	STATUS			  ;Restore STATUS
	swapf	WTEMP,f			  
	swapf	WTEMP,w			  ;Restore W without corrupting STATUS bits
	retfie                    ;Return from interrupt	

;********************************* Start of Program ********************************
Main
	ifndef	__DEBUG
	call    0x3FF      ; retrieve factory calibration value
	else
	movlw	b'10000000'
	endif

	clrwdt
	clrf	GPIO
	BANKSEL	OSCCAL					  
	movwf   OSCCAL      	;Bank1 update register with factory cal value 
	movlw	TRISIOSET		
	movwf	TRISIO			;Bank1 Tri-State Selected Inputs
	clrf	VRCON			;Bank1 Vref Off
	movlw	OPTIONSET
	movwf	OPTION_REG		;Bank1 TIMER0 Prescaler = 4 and pull-ups disabled
	movlw	ANSELSET		
	movwf	ANSEL			;Bank1 Configure AN0 & prescale to A/D	
	bsf		PIE1,ADIE		;Bank1 Use ADC Interrupt

	BANKSEL	0					;BANK 0
;	clrf	OUTREG			;Clear output register
;	movf	OUTREG, w
;	movwf	GPIO			;Set output low to start
	
	movlw	0x07		
	movwf	CMCON			;Comparator Off

	bsf		INTCON,T0IE 	;Interrupt on TIMER0 Overflow Enabled
	bcf		INTCON,T0IF 	;Clear TIMER0 Overflow Interrupt Flag
	bsf		INTCON,PEIE
	bsf		INTCON,GIE		;Turn on Global Interrupts

	movlw	ADCON0SET		
	movwf	ADCON0			;Select AN3, Left justified & enables A/D
	NOP
	NOP
	NOP
	NOP						; Give 4 uS delay before starting A/D

	clrw					;Set W=0
	movwf	AVGLO
	movwf	AVGHI
	movwf	AVGADC
	movwf	TM0CTR
	movwf	AVGCTR
	movwf	FLAGS
	movlw	MAXSAMPLE
	movwf	SAMPLE
	movlw	THRESH1			;Threshold value

;	bsf		ADCON0, GO		;Bank 0, start conversion
	bsf		INTCON,GIE		;Turn on Global Interrupts

	clrf	TMR0			;Bank 0

	call	DisablePWM

;Wait for voltages to settle
	movlw	d'10'			;10 * 10 = 100 mSec
	movwf	DelayCount
	call	Delay10


; *********************************** Main Program Loop ****************************
Loop:

;*********************** Read Vin and loop until voltage OK **************************
ReadVin:
	clrwdt
	movf	AVGADC,W
	sublw	MINVIN				;W = MINVIN-Vin
	btfss	STATUS,C
	goto	CheckVin2			;Borrow: Vin >= MINVIN

VinTooLow:
	call	DisablePWM
	goto	ReadVin				;Loop with PWM off

CheckVin2:						;Vin >= MINVIN
	movf	AVGADC,W
	sublw	MAXVIN				;W = MAXVIN-Vin
	btfsc	STATUS,C
	goto	VinOK				;No Borrow: Vin <= MAXVIN

VinTooHigh:						;Vin > MAXVIN
	call	DisablePWM
	goto	ReadVin

VinOK:
	movlw	D'95'				;Set to 100%
	movwf	PWMvalue
	call	SetPWM

	movlw	d'50'			;10 * 5 = 50 mSec
	movwf	DelayCount
	call	Delay10

	call	EnablePWM

PWM100:
	movlw	d'20'			;10 * 5 = 50 mSec
	movwf	DelayCount
	call	Delay10

PWM10:
	clrwdt
	movlw	D'20'				;Set to 20%
	movwf	PWMvalue
	call	SetPWM

ReadVin10:
	clrwdt
	movf	AVGADC,W
	sublw	5					;W = 5-Vin
	btfss	STATUS,C
	goto	ReadVin10			;Borrow: Vin >= MINVIN

VinTooLow10:
	call	DisablePWM

EndLoop:
	goto	ReadVin				;Wait for voltage to come back up


; *********************** Adjust PWM to value in PWMval **********************
; Use MSB only, so 100% is 168
; Duty Cycle ratio is (CCPR1L + CCP1CON<4:5>)/(4*(PR2+1)
; PR2 = 65, Max PWM = 1024/264
; 

SetPWM:
	movf	PWMvalue,W			;1-99%
	addwf	PWMvalue,W
	movwf	OnTime

	movf	OnTime,W
	sublw	d'200'
	movwf	OffTime
	sublw	0				;W = 0-OffTime
	movwf	OffTime
	movlw	COMP			;Compensation
	addwf	OffTime,F

	movf	OnTime,W
	sublw	0				;W = 0-OnTime
	movwf	OnTime
	movlw	COMP			;Compensation
	addwf	OnTime,F
	return

; *********************** Turn PWM Output OFF and Latch ***************
ShutDownPWM:
	BANK1
	bsf		TRISIO,PWMOUT		;Shut down PWM by tristating output pin
	BANK0
	return

; *********************** Turn PWM Output OFF *************************
DisablePWM:
	BANK1
	bsf		TRISIO,PWMOUT		;Shut down PWM by tristating output pin
	BANK0
	return

; *********************** Turn PWM Output ON **************************
EnablePWM:
	BANK1
	bcf		TRISIO,PWMOUT		;Enable PWM by un-tristating output pin
	BANK0
	return

; ***************** Delay 10 mSec * DelayCount *************************
Delay10:
	movlw	d'12'				;12 counts at 800 uSec = 10 mSec
	movwf	DelayCtr
Delay10a:			;
	clrwdt
	movf	DelayCtr,W
	iorwf	DelayCtr,W
	btfss	STATUS,Z
	goto	Delay10a

	decf	DelayCount, F
	btfss	STATUS,Z
	goto	Delay10

	return						;return when delay = 0

;************** Software PWM ***************
; 4 MHz clock = 1 MHz or 1 uSec instruction cycles
; Full Period can be 200 counts = 200 uSec for 5 kHz, 20 counts ON 180 counts OFF = 10%
; Using PS 1/4, 4 uSec tick, 200 uSec for 1.25 kHz, 20 counts ON 180 counts OFF = 10%
; 
; On TMR0 interrupt
;   If OFF, load TMR0 with 256 - T(on), and turn ON
;   If ON, load TMR0 with 256 - T(off), and turn OFF
; T(on) = DC * 2
; T(off) = 200 - T(on)
; For 10% DC, T(on) = 25, TMR0 = 231, T(off) = 225, TMR0 = 31

	END                       ; directive 'end of program'

